@@ -1,9 +1,9 @@
 /*
  * modsin.c
  *
- * Copyright (c) Chris Putnam 2004-8
+ * Copyright (c) Chris Putnam 2004-2013
  *
- * Source code released under the GPL
+ * Source code released under the GPL version 2
  *
  */
 #include <stdio.h>
@@ -18,6 +18,42 @@
 #include "name.h"
 #include "reftypes.h"
 #include "modstypes.h"
+#include "marc.h"
+#include "bibutils.h"
+#include "modsin.h"
+
+void
+modsin_initparams( param *p, const char *progname )
+{
+
+	p->readformat       = BIBL_MODSIN;
+	p->format_opts      = 0;
+	p->charsetin        = BIBL_CHARSET_UNICODE;
+	p->charsetin_src    = BIBL_SRC_DEFAULT;
+	p->latexin          = 0;
+	p->utf8in           = 1;
+	p->xmlin            = 1;
+	p->nosplittitle     = 0;
+	p->verbose          = 0;
+	p->addcount         = 0;
+	p->singlerefperfile = 0;
+	p->output_raw       = BIBL_RAW_WITHMAKEREFID |
+	                      BIBL_RAW_WITHCHARCONVERT;
+
+	p->readf    = modsin_readf;
+	p->processf = modsin_processf;
+	p->cleanf   = NULL;
+	p->typef    = NULL;
+	p->convertf = NULL;
+	p->all      = NULL;
+	p->nall     = 0;
+
+	list_init( &(p->asis) );
+	list_init( &(p->corps) );
+
+	if ( !progname ) p->progname = NULL;
+	else p->progname = strdup( progname );
+}
 
 static char modsns[]="mods";
 
@@ -26,7 +62,7 @@ modsin_detailr( xml *node, newstr *value )
 {
 	if ( node->value && node->value->len ) {
 		if ( value->len ) newstr_addchar( value, ' ' );
-		newstr_strcat( value, node->value->data );
+		newstr_newstrcat( value, node->value );
 	}
 	if ( node->down ) modsin_detailr( node->down, value );
 	if ( node->next ) modsin_detailr( node->next, value );
@@ -37,8 +73,7 @@ modsin_detail( xml *node, fields *info, int level )
 {
 	newstr type, value, *tp;
 	if ( node->down ) {
-		newstr_init( &type );
-		newstr_init( &value );
+		newstrs_init( &type, &value, NULL );
 		tp = xml_getattrib( node, "type" );
 		if ( tp ) {
 			newstr_newstrcpy( &type, tp );
@@ -48,8 +83,7 @@ modsin_detail( xml *node, fields *info, int level )
 		if ( type.data && !strcasecmp( type.data, "PAGE" ) ) {
 			fields_add( info, "PAGESTART", value.data, level );
 		} else fields_add( info, type.data, value.data, level );
-		newstr_free( &type );
-		newstr_free( &value );
+		newstrs_free( &type, &value, NULL );
 	}
 }
 
@@ -89,7 +123,7 @@ modsin_date( xml *node, fields *info, int level, int part )
 }
 
 static void
-modsin_pager( xml *node, newstr *sp, newstr *ep, newstr *tp )
+modsin_pager( xml *node, newstr *sp, newstr *ep, newstr *tp, newstr *lp )
 {
 	if ( xml_tagexact( node, "start" ) ) {
 		newstr_newstrcpy( sp, node->value );
@@ -97,41 +131,46 @@ modsin_pager( xml *node, newstr *sp, newstr *ep, newstr *tp )
 		newstr_newstrcpy( ep, node->value );
 	} else if ( xml_tagexact( node, "total" ) ) {
 		newstr_newstrcpy( tp, node->value );
+	} else if ( xml_tagexact( node, "list" ) ) {
+		newstr_newstrcpy( lp, node->value );
 	}
-	if ( node->down ) modsin_pager( node->down, sp, ep, tp );
-	if ( node->next ) modsin_pager( node->next, sp, ep, tp );
+	if ( node->down ) modsin_pager( node->down, sp, ep, tp, lp );
+	if ( node->next ) modsin_pager( node->next, sp, ep, tp, lp );
 }
 
 static void
 modsin_page( xml *node, fields *info, int level )
 {
-	newstr sp, ep, tp;
+	newstr sp, ep, tp, lp;
 	if ( node->down ) {
-		newstr_init( &sp );
-		newstr_init( &ep );
-		newstr_init( &tp );
-		modsin_pager( node->down, &sp, &ep, &tp );
+		newstrs_init( &sp, &ep, &tp, &lp, NULL );
+		modsin_pager( node->down, &sp, &ep, &tp, &lp );
 		if ( sp.len || ep.len ) {
 			if ( sp.len )
 				fields_add( info, "PAGESTART", sp.data, level );
 			if ( ep.len )
 				fields_add( info, "PAGEEND", ep.data, level );
+		} else if ( lp.len ) {
+			fields_add( info, "PAGESTART", lp.data, level );
 		}
 		if ( tp.len )
 			fields_add( info, "TOTALPAGES", tp.data, level );
-		newstr_free( &sp );
-		newstr_free( &ep );
-		newstr_free( &tp );
+		newstrs_free( &sp, &ep, &tp, &lp, NULL );
 	}
 }
 
 static void
 modsin_titler( xml *node, newstr *title, newstr *subtitle )
 {
-	if ( xml_tagexact( node, "title" ) )
-		newstr_strcat( title, node->value->data );
-	else if ( xml_tagexact( node, "subTitle" ) )
-		newstr_strcat( subtitle, node->value->data );
+	if ( xml_tagexact( node, "title" ) ) {
+		if ( title->len ) {
+			newstr_strcat( title, " : " );
+			newstr_newstrcat( title, node->value );
+		} else {
+			newstr_newstrcat( title, node->value );
+		}
+	} else if ( xml_tagexact( node, "subTitle" ) )
+		newstr_newstrcat( subtitle, node->value );
 	if ( node->down ) modsin_titler( node->down, title, subtitle );
 	if ( node->next ) modsin_titler( node->next, title, subtitle );
 }
@@ -142,11 +181,10 @@ modsin_title( xml *node, fields *info, int level )
 	newstr title, subtitle;
 	int abbr = xml_tag_attrib( node, "titleInfo", "type", "abbreviated" );
 	if ( node->down ) {
-		newstr_init( &title );
-		newstr_init( &subtitle );
+		newstrs_init( &title, &subtitle, NULL );
 		modsin_titler( node->down, &title, &subtitle );
 		if ( title.len ) {
-			if ( abbr ) 
+			if ( abbr )
 				fields_add( info, "SHORTTITLE", title.data, level );
 			else
 				fields_add( info, "TITLE", title.data, level );
@@ -157,37 +195,70 @@ modsin_title( xml *node, fields *info, int level )
 			else
 				fields_add( info, "SUBTITLE", subtitle.data, level );
 		}
-		newstr_free( &title );
-		newstr_free( &subtitle );
+		newstrs_free( &title, &subtitle, NULL );
 	}
 }
 
-/*
- * find match in roles if newstr has the format of
+/* modsin_marcrole_convert()
  *
- * creator|cre
+ * Map MARC-authority roles for people or organizations associated
+ * with a reference to internal roles.
+ *
+ * Take input strings with roles separated by '|' characters, e.g.
+ * "author" or "author|creator" or "edt" or "editor|edt".
  */
-static int
-modsin_rolesmatch( convert *roles_convert, int nroles, newstr *s )
+static void
+modsin_marcrole_convert( newstr *s, char *suffix, newstr *out )
 {
+	convert roles[] = {
+		{ "author",              "AUTHOR" },
+		{ "aut",                 "AUTHOR" },
+		{ "aud",                 "AUTHOR" },
+		{ "aui",                 "AUTHOR" },
+		{ "aus",                 "AUTHOR" },
+		{ "creator",             "AUTHOR" },
+		{ "cre",                 "AUTHOR" },
+		{ "editor",              "EDITOR" },
+		{ "edt",                 "EDITOR" },
+		{ "degree grantor",      "DEGREEGRANTOR" },
+		{ "dgg",                 "DEGREEGRANTOR" },
+		{ "organizer of meeting","ORGANIZER" },
+		{ "orm",                 "ORGANIZER" },
+		{ "patent holder",       "ASSIGNEE" },
+		{ "pth",                 "ASSIGNEE" }
+	};
+	int nroles = sizeof( roles ) / sizeof( roles[0] );
+	int i, nmismatch, n = -1;
 	char *p, *q;
-	int i, nmismatch;
-	for ( i=0; i<nroles; ++i ) {
-		p = s->data;
-		while ( *p ) {
-			q = roles_convert[i].mods;
-			nmismatch = 0;
-			while ( *p && *p!='|' && nmismatch == 0) {
-				if ( toupper( *p ) != toupper( *q ) )
-					nmismatch++;
-				p++;
-				q++;
+
+	if ( s->len == 0 ) {
+		/* ...default to author on an empty string */
+		n = 0;
+	} else {
+		/* ...find first match in '|'-separated list */
+		for ( i=0; i<nroles && n==-1; ++i ) {
+			p = s->data;
+			while ( *p ) {
+				q = roles[i].mods;
+				nmismatch = 0;
+				while ( *p && *p!='|' && nmismatch == 0) {
+					if ( toupper( (unsigned char)*p ) != toupper( (unsigned char)*q ) )
+						nmismatch++;
+					p++;
+					q++;
+				}
+				if ( !nmismatch && !(*(q++))) n = i;
+				if ( *p=='|' ) p++;
 			}
-			if ( !nmismatch && !(*(q++))) return i;
-			if ( *p=='|' ) p++;
 		}
 	}
-	return -1;
+
+	if ( n!=-1 ) {
+		newstr_strcpy( out, roles[n].internal );
+		if ( suffix ) newstr_strcat( out, suffix );
+	} else {
+		newstr_strcpy( out, s->data );
+	}
 }
 
 static void
@@ -197,119 +268,69 @@ modsin_asis_corp_r( xml *node, newstr *name, newstr *role )
 		newstr_newstrcpy( name, node->value );
 	else if ( xml_tagexact( node, "roleTerm" ) ) {
 		if ( role->len ) newstr_addchar( role, '|' );
-		newstr_strcat( role, node->value->data );
+		newstr_newstrcat( role, node->value );
 	}
 	if ( node->down ) modsin_asis_corp_r( node->down, name, role );
 	if ( node->next ) modsin_asis_corp_r( node->next, name, role );
 }
 
 static void
-modsin_personr( xml *node, newstr *name, newstr *roles )
+modsin_personr( xml *node, newstr *name, newstr *suffix, newstr *roles )
 {
+	newstr outname;
+	newstr_init( &outname );
 	if ( xml_tagexact( node, "namePart" ) ) {
 		if ( xml_tag_attrib( node, "namePart", "type", "family" ) ) {
 			if ( name->len ) newstr_prepend( name, "|" );
 			newstr_prepend( name, node->value->data );
+		} else if (xml_tag_attrib( node, "namePart", "type", "suffix") ||
+		           xml_tag_attrib( node, "namePart", "type", "termsOfAddress" )) {
+			if ( suffix->len ) newstr_addchar( suffix, ' ' );
+			newstr_strcat( suffix, node->value->data );
 		} else if (xml_tag_attrib( node, "namePart", "type", "date")){
 		} else {
 			if ( name->len ) newstr_addchar( name, '|' );
-			if ( strchr( node->value->data, ',' ) ) 
-				name_comma( node->value->data, name );
-			else name_nocomma( node->value->data, name );
+			name_parse( &outname, node->value, NULL, NULL );
+			newstr_newstrcat( name, &outname );
 		}
 	} else if ( xml_tagexact( node, "roleTerm" ) ) {
 		if ( roles->len ) newstr_addchar( roles, '|' );
-		newstr_strcat( roles, node->value->data );
+		newstr_newstrcat( roles, node->value );
 	}
-	if ( node->down ) modsin_personr( node->down, name, roles );
-	if ( node->next ) modsin_personr( node->next, name, roles );
+	if ( node->down ) modsin_personr( node->down, name, suffix, roles );
+	if ( node->next ) modsin_personr( node->next, name, suffix, roles );
+	newstr_free( &outname );
 }
 
 static void
-modsin_asis_corp( xml *node, fields *info, int level, convert *roles_convert,
-		int nroles )
+modsin_asis_corp( xml *node, fields *info, int level, char *suffix )
 {
-	newstr name, roles;
-	int n;
+	newstr name, roles, role_out;
 	xml *dnode = node->down;
 	if ( dnode ) {
-		newstr_init( &name );
-		newstr_init( &roles );
+		newstrs_init( &name, &roles, &role_out, NULL );
 		modsin_asis_corp_r( dnode, &name, &roles );
-		if ( roles.len ) {
-			n = modsin_rolesmatch( roles_convert, nroles, &roles );
-			if ( n!=-1 ) {
-				fields_add( info, roles_convert[n].internal, 
-					name.data, level );
-			} else {
-				fields_add( info, roles.data, name.data, level);
-			}
-		}
-		else fields_add( info, roles_convert[0].internal, name.data, 
-			level );
-		newstr_free( &name );
-		newstr_free( &roles );
+		modsin_marcrole_convert( &roles, suffix, &role_out );
+		fields_add( info, role_out.data, name.data, level );
+		newstrs_free( &name, &roles, &role_out, NULL );
 	}
 }
 
-static void
-modsin_asis( xml *node, fields *info, int level )
-{
-	convert roles_convert[] = {
-		{ "author",              "AUTHOR:ASIS" },
-		{ "creator",             "AUTHOR:ASIS" },
-		{ "editor",              "EDITOR:ASIS" },
-		{ "degree grantor",      "DEGREEGRANTOR:ASIS" },
-		{ "organizer of meeting","ORGANIZER:ASIS" },
-		{ "patent holder",       "ASSIGNEE:ASIS" }
-	};
-	int nroles = sizeof( roles_convert ) / sizeof( roles_convert[0] );
-	modsin_asis_corp( node, info, level, roles_convert, nroles );
-}
-
-static void
-modsin_corp( xml *node, fields *info, int level )
-{
-	convert roles_convert[] = {
-		{ "author",              "AUTHOR:CORP" },
-		{ "creator",             "AUTHOR:CORP" },
-		{ "editor",              "EDITOR:CORP" },
-		{ "degree grantor",      "DEGREEGRANTOR:CORP" },
-		{ "organizer of meeting","ORGANIZER:CORP" },
-		{ "patent holder",       "ASSIGNEE:CORP" }
-	};
-	int nroles = sizeof( roles_convert ) / sizeof( roles_convert[0] );
-	modsin_asis_corp( node, info, level, roles_convert, nroles );
-}
-
 static void
 modsin_person( xml *node, fields *info, int level )
 {
-	newstr name, roles, role;
-	char *p;
+	newstr name, suffix, roles, role, role_out;
 	xml *dnode = node->down;
 	if ( dnode ) {
-		newstr_init( &name );
-		newstr_init( &role );
-		newstr_init( &roles );
-		modsin_personr( dnode, &name, &roles );
-		/* no defined role, default to author */
-		if ( !roles.len ) newstr_strcpy( &roles, "author" );
-		p = roles.data;
-		while ( p && *p ) {
-			while ( p && *p && *p!='|' ) newstr_addchar(&role,*p++);
-			if ( !strcasecmp( role.data, "author" ) ||
-			     !strcasecmp( role.data, "creator" ) )
-				fields_add( info, "AUTHOR", name.data, level );
-			else if ( !strcasecmp( role.data, "editor" ) )
-				fields_add( info, "EDITOR", name.data, level );
-			else fields_add( info, role.data, name.data, level );
-			if ( *p=='|' ) p++;
-			newstr_empty( &role );
+		newstrs_init( &name, &suffix, &role, &roles, &role_out, NULL );
+		modsin_personr( dnode, &name, &suffix, &roles );
+		modsin_marcrole_convert( &roles, NULL, &role_out );
+		if ( suffix.len ) {
+			newstr_strcat( &name, "||" );
+			newstr_newstrcat( &name, &suffix );
 		}
-		newstr_free( &name );
-		newstr_free( &roles );
-		newstr_free( &role );
+		fields_add( info, role_out.data, name.data, level );
+		newstrs_free( &name, &suffix, &role, &roles, NULL );
 	}
 }
 
@@ -333,7 +354,7 @@ modsin_placeterm( xml *node, fields *info, int level, int school )
 			type = xml_getattrib( node, "authority" );
 			if ( type && type->len ) newstr_newstrcpy(&s, type);
 			newstr_addchar( &s, '|' );
-			newstr_strcat( &s, node->value->data );
+			newstr_newstrcat( &s, node->value );
 			fields_add( info, addresscode_tag, s.data, level );
 			newstr_free( &s );
 		}
@@ -358,11 +379,11 @@ modsin_origininfor( xml *node, fields *info, int level, newstr *pub, newstr *add
 	if ( xml_tagexact( node, "dateIssued" ) )
 		modsin_date( node, info, level, 0 );
 	else if ( xml_tagexact( node, "publisher" ) )
-		newstr_strcat( pub, node->value->data );
+		newstr_newstrcat( pub, node->value );
 	else if ( xml_tagexact( node, "edition" ) )
-		newstr_strcat( ed, node->value->data );
+		newstr_newstrcat( ed, node->value );
 	else if ( xml_tagexact( node, "issuance" ) )
-		newstr_strcat( iss, node->value->data );
+		newstr_newstrcat( iss, node->value );
 	else if ( xml_tagexact( node, "place" ) )
 		modsin_placer( node, info, level, 0 );
 	if ( node->down )
@@ -376,12 +397,9 @@ modsin_origininfo( xml *node, fields *info, int level )
 {
 	newstr publisher, address, addcode, edition, issuance;
 	if ( node->down ) {
-		newstr_init( &publisher );
-		newstr_init( &address );
-		newstr_init( &addcode );
-		newstr_init( &edition );
-		newstr_init( &issuance );
-		modsin_origininfor( node->down, info, level, &publisher, &address, &addcode, &edition, &issuance );
+		newstrs_init( &publisher, &address, &addcode, &edition, &issuance, NULL );
+		modsin_origininfor( node->down, info, level, &publisher, 
+				&address, &addcode, &edition, &issuance );
 		if ( publisher.len )
 			fields_add( info, "PUBLISHER", publisher.data, level );
 		if ( address.len )
@@ -392,11 +410,7 @@ modsin_origininfo( xml *node, fields *info, int level )
 			fields_add( info, "EDITION", edition.data, level );
 		if ( issuance.len ) 
 			fields_add( info, "ISSUANCE", issuance.data, level );
-		newstr_free( &publisher );
-		newstr_free( &address );
-		newstr_free( &addcode );
-		newstr_free( &edition );
-		newstr_free( &issuance );
+		newstrs_free( &publisher, &address, &addcode, &edition, &issuance, NULL );
 	}
 }
 
@@ -429,30 +443,6 @@ modsin_id1( xml *node, fields *info, int level )
 static void
 modsin_genre( xml *node, fields *info, int level )
 {
-       char *marc[] = { "abstract or summary", "art original",
-		"art reproduction", "atlas", "autobiography",
-		"bibliography", "book", "catalog", "chart", "comic strip",
-		"conference publication", "database", "dictionary",
-		"diorama", "directory", "discography", "drama",
-		"encyclopedia", "essay", "festschrift", "fiction",
-		"filmography", "filmstrip", "flash card", "folktale",
-		"font", "game", "government publication", "graphic",
-		"globe", "handbook", "history", "humor, satire",
-		"index", "instruction", "interview", "kit",
-		"language instruction", "law report or digest",
-		"legal article", "legal case and case notes", "legislation",
-		"letter", "loose-leaf", "map", "memoir", "microscope slide",
-		"model", "motion picture", "multivolume monograph",
-		"newspaper", "novel", "numeric data",
-		"online system or service", "patent", "periodical",
-		"picture", "poetry", "programmed text", "realia",
-		"rehersal", "remote sensing image", "reporting",
-		"review", "series", "short story", "slide", "sound",
-		"speech", "statistics", "survey of literature",
-		"technical drawing", "technical report", "theses",
-		"toy", "transparency", "treaty", "videorecording",
-		"web site"
-	};
 	char *added[] = { "manuscript", "academic journal", "magazine",
 		"hearing", "report", "Ph.D. thesis", "Masters thesis",
 		"Diploma thesis", "Doctoral thesis", "Habilitation thesis",
@@ -460,14 +450,10 @@ modsin_genre( xml *node, fields *info, int level )
 		"teletype", "airtel", "memo", "e-mail communication",
 		"press release", "television broadcast", "electronic"
 	};
-	int nmarc = sizeof( marc ) / sizeof( char* );
 	int nadded = sizeof( added ) /sizeof( char *);
 	int j, ismarc = 0, isadded = 0;
 	if ( node->value && node->value->len ) {
-		for ( j=0; j<nmarc && ismarc==0; ++j ) {
-			if ( !strcasecmp( node->value->data, marc[j] ) )
-				ismarc = 1;
-		}
+		if ( marc_findgenre( node->value->data )!=-1 ) ismarc = 1;
 		for ( j=0; j<nadded && ismarc==0 && isadded==0; ++j ) {
 			if ( !strcasecmp( node->value->data, added[j] ) )
 				isadded = 1;
@@ -486,11 +472,25 @@ modsin_resource( xml *node, fields *info, int level )
 		fields_add( info, "RESOURCE", node->value->data, level );
 }
 
+static void
+modsin_languager( xml *node, fields *info, int level )
+{
+	if ( xml_tag_attrib( node, "languageTerm", "type", "text" ) ) {
+		if ( node->value && node->value->len )
+			fields_add( info, "LANGUAGE", node->value->data, level );
+	}
+	if ( node->next ) modsin_languager( node->next, info, level );
+}
+
 static void
 modsin_language( xml *node, fields *info, int level )
 {
+	/* Old versions of MODS had <language>English</language> */
 	if ( node->value && node->value->len )
 		fields_add( info, "LANGUAGE", node->value->data, level );
+
+	/* New versions of MODS have <language><languageTerm>English</languageTerm></language> */
+	if ( node->down ) modsin_languager( node->down, info, level );
 }
 
 static void
@@ -507,6 +507,13 @@ modsin_note( xml *node, fields *info, int level )
 		fields_add( info, "NOTES", node->value->data, level );
 }
 
+static void
+modsin_annote( xml *node, fields *info, int level )
+{
+	if ( node->value && node->value->len )
+		fields_add( info, "ANNOTE", node->value->data, level );
+}
+
 static void
 modsin_abstract( xml *node, fields *info, int level )
 {
@@ -517,16 +524,23 @@ modsin_abstract( xml *node, fields *info, int level )
 static void
 modsin_locationr( xml *node, fields *info, int level )
 {
-	char url[]="URL", school[]="SCHOOL", loc[]="LOCATION", *tag=NULL;
-	if ( xml_tagexact( node, "url" ) ) {
+	char url[]="URL", school[]="SCHOOL", loc[]="LOCATION";
+	char fileattach[]="FILEATTACH", *tag=NULL;
+
+	if ( xml_tag_attrib( node, "url", "access", "raw object" ) ) {
+		tag = fileattach;
+	} else if ( xml_tagexact( node, "url" ) ) {
 		tag = url;
 	}
+
 	if ( xml_tag_attrib( node, "physicalLocation", "type", "school" ) ) {
 		tag = school;
 	} else if ( xml_tagexact( node, "physicalLocation" ) ) {
 		tag = loc;
 	}
+
 	if ( tag ) fields_add( info, tag, node->value->data, level );
+
 	if ( node->down ) modsin_locationr( node->down, info, level );
 	if ( node->next ) modsin_locationr( node->next, info, level );
 }
@@ -540,8 +554,10 @@ modsin_location( xml *node, fields *info, int level )
 static void
 modsin_descriptionr( xml *node, newstr *s )
 {
-	if ( xml_tagexact( node, "extent" ) ) 
+	if ( xml_tagexact( node, "extent" ) ||
+	     xml_tagexact( node, "note" ) ) {
 		newstr_newstrcpy( s, node->value );
+	}
 	if ( node->down ) modsin_descriptionr( node->down, s );
 	if ( node->next ) modsin_descriptionr( node->next, s );
 }
@@ -593,6 +609,22 @@ modsin_classification( xml *node, fields *info, int level )
 	if ( node->down ) modsin_classification( node->down, info, level );
 }
 
+static void
+modsin_recordinfo( xml *node, fields *info, int level )
+{
+	xml *curr;
+
+	/* extract recordIdentifier */
+	curr = node;
+	while ( curr ) {
+		if ( xml_tagexact( curr, "recordIdentifier" ) ) {
+			fields_add( info, "REFNUM", curr->value->data, level );
+		}
+		curr = curr->next;
+	}
+
+}
+
 static void
 modsin_identifier( xml *node, fields *info, int level )
 {
@@ -603,12 +635,15 @@ modsin_identifier( xml *node, fields *info, int level )
 		{ "doi",           "DOI"          },
 		{ "url",           "URL"          },
 		{ "uri",           "URL"          },
-		{ "pubmed",        "PUBMED"       },
+		{ "pmid",          "PMID"         },
+		{ "pubmed",        "PMID"         },
 		{ "medline",       "MEDLINE"      },
+		{ "arXiv",         "ARXIV"        },
 		{ "pii",           "PII"          },
 		{ "isi",           "ISIREFNUM"    },
 		{ "serial number", "SERIALNUMBER" },
-		{ "accessnum",     "ACCESSNUM"    }
+		{ "accessnum",     "ACCESSNUM"    },
+		{ "jstor",         "JSTOR"        },
 	};
 	int i , n = sizeof( ids ) / sizeof( ids[0] );
 	if ( !node->value || !node->value->data ) return;
@@ -626,9 +661,11 @@ modsin_mods( xml *node, fields *info, int level )
 	else if ( xml_tag_attrib( node, "name", "type", "personal" ) )
 		modsin_person( node, info, level );
 	else if ( xml_tag_attrib( node, "name", "type", "corporate" ) )
-		modsin_corp( node, info, level );
+		modsin_asis_corp( node, info, level, ":CORP" );
 	else if ( xml_tagexact( node, "name" ) )
-		modsin_asis( node, info, level );
+		modsin_asis_corp( node, info, level, ":ASIS" );
+	else if ( xml_tagexact( node, "recordInfo" ) && node->down )
+		modsin_recordinfo( node->down, info, level );
 	else if  ( xml_tagexact( node, "part" ) )
 		modsin_part( node, info, level );
 	else if ( xml_tagexact( node, "identifier" ) )
@@ -647,6 +684,8 @@ modsin_mods( xml *node, fields *info, int level )
 		modsin_date( node, info, level, 0 );
 	else if ( xml_tagexact( node, "note" ) )
 		modsin_note( node, info, level );
+	else if ( xml_tagexact( node, "bibtex-annote" ) )
+		modsin_annote( node, info, level );
 	else if ( xml_tagexact( node, "abstract" ) )
 		modsin_abstract( node, info, level );
 	else if ( xml_tagexact( node, "subject" ) )
@@ -658,7 +697,7 @@ modsin_mods( xml *node, fields *info, int level )
 	else if ( xml_tagexact( node, "physicalDescription" ) )
 		modsin_description( node, info, level );
 	else if ( xml_tag_attrib( node, "relatedItem", "type", "host" ) ||
-	          xml_tag_attrib( node, "relatedItem", "type", "series" ) ) {
+		  xml_tag_attrib( node, "relatedItem", "type", "series" ) ) {
 		if ( node->down ) modsin_mods( node->down, info, level+1 );
 	}
 
@@ -676,16 +715,6 @@ modsin_assembleref( xml *node, fields *info )
 	if ( node->next ) modsin_assembleref( node->next, info );
 }
 
-void
-modsin_convertf( fields *modsin, fields *info, int reftype, int verbose, 
-	variants *all, int nall )
-{
-	int i;
-	for ( i=0; i<modsin->nfields; ++i )
-		fields_add( info, modsin->tag[i].data, modsin->data[i].data,
-				modsin->level[i] );
-}
-
 int
 modsin_processf( fields *modsin, char *data, char *filename, long nref )
 {
@@ -730,7 +759,7 @@ modsin_readf( FILE *fp, char *buf, int bufsize, int *bufpos, newstr *line,
 	newstr_init( &tmp );
 
 	do {
-		if ( line->data ) newstr_strcat( &tmp, line->data );
+		if ( line->data ) newstr_newstrcat( &tmp, line );
 		if ( tmp.data ) {
 			m = xml_getencoding( &tmp );
 			if ( m!=CHARSET_UNKNOWN ) file_charset = m;
