@@ -1,12 +1,11 @@
 /*
  * newstr_conv.c
  *
- * Copyright (c) Chris Putnam 1999-2008
+ * Copyright (c) Chris Putnam 1999-2013
  *
- * Source code released under the GPL
+ * Source code released under the GPL version 2
  *
- * newstring routines for dynamically allocated strings
- * conversions between character sets
+ * newstring routines for converting newstrings between character sets
  *
  */
 #include <stdio.h>
@@ -18,34 +17,9 @@
 #include "latex.h"
 #include "entities.h"
 #include "utf8.h"
-/* #include "gb18030.h" */
-#include "newstr_conv.h"
-
+#include "gb18030.h"
 #include "charsets.h"
-
-int
-get_charset( char *name )
-{
-	int i;
-	if ( name==NULL ) return CHARSET_UNKNOWN;
-	for ( i=0; i<nallcharconvert; ++i ){
-		if ( !strcasecmp( name, allcharconvert[i].name ) ) return i;
-		else if ( allcharconvert[i].name2[0]!='\0' &&
-			!strcasecmp( name, allcharconvert[i].name2 ) ) return i;
-	}
-	return CHARSET_UNKNOWN;
-}
-
-void
-list_charsets( FILE *fp )
-{
-	int i;
-	for ( i=0; i<nallcharconvert; ++i ){
-		fprintf( fp, " %s", allcharconvert[i].name );
-		if ( (i>0 && i%5==0) || (i==nallcharconvert-1) )
-			fprintf( fp, "\n");
-	}
-}
+#include "newstr_conv.h"
 
 static void
 addentity( newstr *s, unsigned int ch )
@@ -90,7 +64,6 @@ addutf8char( newstr *s, unsigned int ch, int xmlout, int utf8out )
 		newstr_addchar( s, code[i] );
 }
 
-#if 0
 static void
 addgb18030char( newstr *s, unsigned int ch, int xmlout, int utf8out )
 {
@@ -105,7 +78,6 @@ addgb18030char( newstr *s, unsigned int ch, int xmlout, int utf8out )
 	for ( i=0; i<nc; ++i )
 		newstr_addchar( s, code[i] );
 }
-#endif
 
 static void
 addlatexchar( newstr *s, unsigned int ch, int xmlout, int utf8out )
@@ -123,24 +95,6 @@ addlatexchar( newstr *s, unsigned int ch, int xmlout, int utf8out )
 	}
 }
 
-static unsigned int
-lookupchar( int charsetin, char c )
-{
-	return allcharconvert[charsetin].table[(int)c];
-}
-
-static unsigned int
-lookupuni( int charsetout, unsigned int unicode )
-{
-	int i;
-	if ( charsetout==CHARSET_UNICODE ) return unicode;
-	for ( i=0; i<256; ++i ) {
-		if ( unicode == allcharconvert[charsetout].table[i] )
-			return i;
-	}
-	return '?';
-}
-
 /*
  * get_unicode()
  * 
@@ -169,8 +123,7 @@ get_unicode( newstr *s, unsigned int *pi, int charsetin, int latexin, int utf8in
 	if ( xmlin && s->data[*pi]=='&' ) {
 		ch = decode_entity( s->data, pi, &unicode, &err );
 	} else if ( charsetin==CHARSET_GB18030 ) {
-		ch = ( unsigned int ) s->data[*pi];
-/*		ch = gb18030_decode( s->data, pi );*/
+		ch = gb18030_decode( s->data, pi );
 		unicode = 1;
 	} else if ( latexin ) {
 		/* Must handle bibtex files in UTF8/Unicode */
@@ -186,42 +139,59 @@ get_unicode( newstr *s, unsigned int *pi, int charsetin, int latexin, int utf8in
 		*pi = *pi + 1;
 	}
 	if ( !unicode && charsetin!=CHARSET_UNICODE )
-		ch = lookupchar( charsetin, ch );
+		ch = charset_lookupchar( charsetin, ch );
 	return ch;
 }
 
-static void
+static int
 write_unicode( newstr *s, unsigned int ch, int charsetout, int latexout,
 		int utf8out, int xmlout )
 {
 	unsigned int c;
-	if ( utf8out ) addutf8char( s, ch, xmlout, utf8out );
-	else if ( latexout ) addlatexchar( s, ch, xmlout, utf8out );
-/*	else if ( charsetout==CHARSET_GB18030 ) addgb18030char( s, ch, xmlout, utf8out );*/
-	else {
-		c = lookupuni( charsetout, ch );
+	if ( latexout ) {
+		addlatexchar( s, ch, xmlout, utf8out );
+	} else if ( utf8out ) {
+		addutf8char( s, ch, xmlout, utf8out );
+	} else if ( charsetout==CHARSET_GB18030 ) {
+		addgb18030char( s, ch, xmlout, utf8out );
+	} else {
+		c = charset_lookupuni( charsetout, ch );
 		if ( xmlout ) addxmlchar( s, c );
 		else newstr_addchar( s, c );
 	}
+	return 1;
 }
 
-void
-newstr_convert( newstr *s, 
+/*
+ * Returns 1 on memory error condition
+ */
+int
+newstr_convert( newstr *s,
 	int charsetin,  int latexin,  int utf8in,  int xmlin,
 	int charsetout, int latexout, int utf8out, int xmlout )
 {
+	unsigned int pos = 0;
 	unsigned int ch;
 	newstr ns;
-	unsigned int pos = 0;
-	if ( s->len==0 ) return;
+	int ok;
+
+	if ( !s || s->len==0 ) return 1;
+
 	newstr_init( &ns );
+
 	if ( charsetin==CHARSET_UNKNOWN ) charsetin = CHARSET_DEFAULT;
 	if ( charsetout==CHARSET_UNKNOWN ) charsetout = CHARSET_DEFAULT;
+
 	while ( s->data[pos] ) {
 		ch = get_unicode( s, &pos, charsetin, latexin, utf8in, xmlin );
-		write_unicode( &ns, ch, charsetout, latexout, utf8out, xmlout );
+		ok = write_unicode( &ns, ch, charsetout, latexout, utf8out, xmlout );
+		if ( !ok ) return 0;
 	}
+
 	newstr_swapstrings( s, &ns );
+
 	newstr_free( &ns );
+
+	return 1;
 }
 
